#!/usr/bin/python
#############################################################################
# Name:         utility.py
# Purpose:      SSH Connectivity Adapter for Various Purposes
# Maintainer:   Torsten Kurbad
# Copyright:    (c) 2017 iwm-kmrc.de KMRC - Knowledge Media Research Center
# Licence:      GPL
#############################################################################
""" SSH Connectivity Adapter (implementation)

"""

import paramiko

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from persistent import Persistent

from zope.interface import implements
from zope.component import adapts
from zope.component import getSiteManager
from zope.component import queryUtility

from zope.schema.fieldproperty import FieldProperty

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('sshadapter')

from zope.app.component.vocabulary import UtilityVocabulary
from zope.app.container.contained import Contained

from interfaces import ISSHAdapter

DEFAULT_SSH_ADAPTER_NAME = u'SSHAdapter'

class SSHAdapter(Persistent, Contained):
    """ Make SSH Connectivity Adapter Configuration Persistent in ZODB.
    
    """

    implements(ISSHAdapter)

    hostName = FieldProperty(ISSHAdapter['hostName'])
    port = FieldProperty(ISSHAdapter['port'])
    sshUser = FieldProperty(ISSHAdapter['sshUser'])
    sshUserPassword = FieldProperty(ISSHAdapter['sshUserPassword'])
    sshUserKey = FieldProperty(ISSHAdapter['sshUserKey'])

    _autoRegister = False

    def __init__(self, hostName = None, sshUser = None,
        sshUserPassword = None, sshUserKey = None):
        """ Initialize sshadapter with default values. """
        super(SSHAdapter, self).__init__()

        if hostName is not None:
            self.hostName = hostName
        if sshUser is not None:
            self.sshUser = sshUser
        if sshUserPassword is not None:
            self.sshUserPassword = sshUserPassword
        if sshUserKey is not None:
            self.sshUserKey = sshUserKey

    def getAutoRegister(self):
        """ Return autoRegister status. """
        return self._autoRegister

    def setAutoRegister(self, value):
        """ Register the adapter as utility. """
        if value:
            siteManager = getSiteManager(self)

            if queryUtility(ISSHAdapter, name=DEFAULT_SSH_ADAPTER_NAME) is None:
                siteManager.registerUtility(self, ISSHAdapter, DEFAULT_SSH_ADAPTER_NAME)

        self._autoRegister = value

    autoRegister = property(getAutoRegister, setAutoRegister)

    def executeSSHCommand(self, command, stdInData=None, timeout=None):
        """ Execute a command remotely through SSH on the configured
            remote system.
        """

        if self.sshUserKey is None and self.sshUserPassword is None:
            raise paramiko.PasswordRequiredException(
                _(u'Can not open SSH connection to %s. Neither password '
                  u'nor private SSH key have been configured for the SSH '
                  u' user.' % self.hostName))

        # Initialize SSHClient object
        ssh = paramiko.client.SSHClient()

        # Accept All Host Keys
        ssh.set_missing_host_key_policy(paramiko.client.AutoAddPolicy())

        # Evaluate private key and password
        pkey = None
        password = None
        if (self.sshUserKey is not None) and (self.sshUserKey != u''):
            pkeyText = self.sshUserKey
            if pkeyText.startswith('-----BEGIN DSA PRIVATE KEY-----'):
                pkeyObj = paramiko.dsskey.DSSKey.generate()
            elif pkeyText.startswith('-----BEGIN RSA PRIVATE KEY-----'):
                pkeyObj = paramiko.rsakey.RSAKey.generate(1024)
            elif pkeyText.startswith('-----BEGIN EC PRIVATE KEY-----'):
                pkeyObj = paramiko.ecdsakey.ECDSAKey.generate()
            else:
                raise paramiko.SSHException(_(u'Invalid SSH Private Key format for SSH user.'))

            pkeyContent = StringIO(pkeyText)
            pkey = pkeyObj.from_private_key(pkeyContent)

        if (self.sshUserPassword is not None) and (self.sshUserPassword != u''):
            password = self.sshUserPassword

        # Open a connection
        ssh.connect(self.hostName,
                    port = self.port,
                    username = self.sshUser,
                    password = password,
                    pkey = pkey,
                    timeout = timeout,
                    allow_agent = False,
                    look_for_keys = False)

        # Execute command
        stdin, stdout, stderr = ssh.exec_command(command, timeout=timeout)

        # Write to stdin if applicable
        if stdInData is not None:
            stdin.write(stdin)

        stdOutData = stdout.read(32768)
        stdErrData = stderr.read(32768)

        # Close the connection
        ssh.close()

        # Return result of command execution as tuple of strings (stdout, stderr)
        return stdOutData, stdErrData


class SSHAdapterVocabulary(UtilityVocabulary):
    interface = ISSHAdapter
    nameOnly = True
